@@ -319,12 +319,27 @@ class OvsHost(linux.LinuxDevice):
         tmp = self.cmd("%s -daemonize" %(cmd))
         #val = os.system(cmd + "&")
 
+    def add_vtep (self, bridge_name, vtep, *args):
+        '''
+        Add some flows remote devices(*args): 从本主机上到达所有*args的包，全
+        部经由vtep
+        '''
+        if self["fail-mode"] != "secure":
+            #Don't need configure vtep explicitly in standalone mode.
+            return
+        for arg in args:
+            ipaddr = ipaddress.ip_interface(arg["ip"]).ip
+            vxlan_port = self.ovs_get_interface(vtep["name"], "ofport")
+            self.ofctl_add_flow(bridge_name,
+                    "priority=65535,arp,arp_tpa=%s,actions=output:%s"
+                    %(ipaddr, vxlan_port),
+                    "priority=60000,ip,nw_dst=%s,actions=output:%s" 
+                    %(ipaddr, vxlan_port))
+
     def add_gateway (self, bridge_name, gw, *args):
         '''
         Add some flows to ofproto
         '''
-        vxlan_port = None # Get the vxlan at first.
-        #print("devices: %s" %(self.devices))
         #print("mac: %s" %(self.macs))
         if (gw["mac"] not in self.macs):
             # gw is not on this node, Add it as if it's on this node.
@@ -336,43 +351,22 @@ class OvsHost(linux.LinuxDevice):
                         "".join(f"{i:02x}" for i in ipaddr.packed)),
                     'priority=1,icmp,nw_dst=%s,icmp_type=8,icmp_code=0,actions=push:"NXM_OF_ETH_SRC[]",push:"NXM_OF_ETH_DST[]",pop:"NXM_OF_ETH_SRC[]",pop:"NXM_OF_ETH_DST[]",push:"NXM_OF_IP_SRC[]",push:"NXM_OF_IP_DST[]",pop:"NXM_OF_IP_SRC[]",pop:"NXM_OF_IP_DST[]",load:"0xff->NXM_NX_IP_TTL[]",load:"0->NXM_OF_ICMP_TYPE[]",in_port' %(ipaddr))
 
-        for d in args:
-            if d.get("type", None) == "vxlan":
-                vxlan_port = self.ovs_get_interface(d["name"], "ofport")
-                break
-
         for d in args:
             #self.log("d: %s" %(d))
-            if d.get("type", None) != "vxlan":
-                if ((d["mac"] not in self.macs) and (not vxlan_port)):
-                    # d is not on host, but there is no vxlan given.
-                    self.log("Error: there is no vxlan to remote device: %s!"
-                            %(d["name"]), bg_color = "red")
-                    return
-
-                #print("d: %s" %(d))
+            if d.get("type", None) not in ["vm", "container"]:
+                self.log("Don't support device type: %s" %(d.get("type", None)))
+                continue
+            if (d["mac"] in self.macs):
+                # ONly change the mac and ttl on the same host.
                 self.ofctl_add_flow(bridge_name,
                     "priority=10,ip,nw_dst=%s,action=mod_dl_src:%s,"
                     "mod_dl_dst:%s,dec_ttl,output:%s"
-                    %(d["ip"].split("/")[0], gw["mac"], d["mac"],
-                        (d["mac"] in self.macs) and d["ofport"] or vxlan_port))
-
-    def add_vtep (self, bridge_name, vtep, *args):
-        '''
-        Add some flows remote devices(*args): 从本主机上到达所有*args的包，全
-        部经由vtep
-        '''
-        if self["fail-mode"] != "secure":
-            #Don't need configure vtep explicitly in standalone mode.
-            return
-        for arg in args:
-            ip = ipaddress.ip_interface(arg["ip"])
-            vxlan_port = self.ovs_get_interface(vtep["name"], "ofport")
-            self.ofctl_add_flow(bridge_name,
-                    "priority=65535,arp,arp_tpa=%s,actions=output:%s"
-                    %(ip.ip, vxlan_port),
-                    "priority=60000,ip,nw_src=%s,nw_dst=%s,actions=output:%s" 
-                    %(ip.network, ip.ip, vxlan_port))
+                    %(d["ip"].split("/")[0], gw["mac"], d["mac"], d["ofport"]))
+            #else:
+            #    # It's on the other host, only forward
+            #    self.ofctl_add_flow(bridge_name,
+            #        "priority=10,ip,nw_dst=%s,action=output:%s"
+            #        %(d["ip"].split("/")[0], vxlan_port))
 
     def set_out_interface (self, bridge_name, out_if, *args):
         '''
@@ -383,13 +377,13 @@ class OvsHost(linux.LinuxDevice):
             #Don't need configure out_if explicitly in standalone mode.
             return
         for arg in args:
-            ip = ipaddress.ip_interface(arg["ip"])
+            ipaddr = ipaddress.ip_interface(arg["ip"]).ip
             vxlan_port = self.ovs_get_interface(out_if["name"], "ofport")
             self.ofctl_add_flow(bridge_name,
                     "priority=65535,arp,arp_tpa=%s,actions=output:%s"
-                    %(ip.ip, vxlan_port),
-                    "priority=60000,ip,nw_src=%s,nw_dst=%s,actions=output:%s" 
-                    %(ip.network, ip.ip, vxlan_port))
+                    %(ipaddr, vxlan_port),
+                    "priority=60000,ip,nw_dst=%s,actions=output:%s" 
+                    %(ipaddr, vxlan_port))
 
     def del_br (self, *args, **kwargs):
         '''
